require 'spec_helper'
require 'pinpoint/address'

describe 'Pinpoint::Address' do
  let(:address) { Pinpoint::Address.new }

  it { expect(address.respond_to?(:region)).to        eql true }
  it { expect(address.respond_to?(:region=)).to       eql true }
  it { expect(address.respond_to?(:province)).to      eql true }
  it { expect(address.respond_to?(:province=)).to     eql true }
  it { expect(address.respond_to?(:locality)).to      eql true }
  it { expect(address.respond_to?(:locality=)).to     eql true }
  it { expect(address.respond_to?(:postal_code)).to   eql true }
  it { expect(address.respond_to?(:postal_code=)).to  eql true }
  it { expect(address.respond_to?(:zip)).to           eql true }
  it { expect(address.respond_to?(:zip=)).to          eql true }
  it { expect(address.respond_to?(:district)).to      eql true }
  it { expect(address.respond_to?(:district=)).to     eql true }

  describe '#new' do
    context 'when it is passed a set of initial values as a Hash' do
      let(:address) do
        Pinpoint::Address.new(
          name:      'The Frist Center for the Visual Arts',
          street:    '919 Broadway',
          city:      'Nashville',
          state:     'Tennessee',
          county:    'Davidson',
          zip_code:  '37203',
          country:   'United States',
          latitude:  '36.15885623029150',
          longitude: '-86.78226636970851',
        )
      end

      it 'sets its values properly' do
        expect(address.name).to       eql 'The Frist Center for the Visual Arts'
        expect(address.street).to     eql '919 Broadway'
        expect(address.city).to       eql 'Nashville'
        expect(address.state).to      eql 'Tennessee'
        expect(address.county).to     eql 'Davidson'
        expect(address.zip_code).to   eql '37203'
        expect(address.country).to    eql 'United States'
        expect(address.latitude).to   eql '36.15885623029150'
        expect(address.longitude).to  eql '-86.78226636970851'
      end
    end
  end

  describe '#complete?' do
    context 'when any of the main parts of the address are blank' do
      let(:address) do
        Pinpoint::Address.new(
          street:   '',
          city:     'Nashville',
          state:    'Tennessee',
          zip_code: '37203',
          country:  'US',
        )
      end

      it 'is false' do
        expect(address).not_to be_complete
      end
    end

    context 'when any of the main parts of the address are missing' do
      let(:address) do
        Pinpoint::Address.new(
          street:   nil,
          city:     'Nashville',
          state:    'Tennessee',
          zip_code: '37203',
          country:  'US',
        )
      end

      it 'is false' do
        expect(address).not_to be_complete
      end
    end

    context 'when all of the main parts of the address are present' do
      let(:address) do
        Pinpoint::Address.new(
          street:   '919 Broadway',
          city:     'Nashville',
          state:    'Tennessee',
          zip_code: '37203',
          country:  'US',
        )
      end

      it 'is true' do
        expect(address).to be_complete
      end
    end

    context 'when all of the main parts of the address are missing' do
      let(:address) do
        Pinpoint::Address.new(
          street:   nil,
          city:     nil,
          state:    nil,
          zip_code: nil,
          country:  nil,
        )
      end

      it 'is false' do
        expect(address).not_to be_complete
      end
    end
  end

  describe '#incomplete?' do
    context 'when any of the main parts of the address are blank' do
      let(:address) do
        Pinpoint::Address.new(
          street:   '',
          city:     'Nashville',
          state:    'Tennessee',
          zip_code: '37203',
          country:  'US',
        )
      end

      it 'is true' do
        expect(address).to be_incomplete
      end
    end

    context 'when any of the main parts of the address are missing' do
      let(:address) do
        Pinpoint::Address.new(
          street:   nil,
          city:     'Nashville',
          state:    'Tennessee',
          zip_code: '37203',
          country:  'US',
        )
      end

      it 'is true' do
        expect(address).to be_incomplete
      end
    end

    context 'when all of the main parts of the address are present' do
      let(:address) do
        Pinpoint::Address.new(
          street:   '919 Broadway',
          city:     'Nashville',
          state:    'Tennessee',
          zip_code: '37203',
          country:  'US',
        )
      end

      it 'is false' do
        expect(address).not_to be_incomplete
      end
    end

    context 'when all of the main parts of the address are missing' do
      let(:address) do
        Pinpoint::Address.new(
          street:   nil,
          city:     nil,
          state:    nil,
          zip_code: nil,
          country:  nil,
        )
      end

      it 'is false' do
        expect(address).not_to be_incomplete
      end
    end
  end

  describe '#empty?' do
    context 'when any of the main parts of the address are blank' do
      let(:address) do
        Pinpoint::Address.new(
          street:   '',
          city:     'Nashville',
          state:    'Tennessee',
          zip_code: '37203',
          country:  'US',
        )
      end

      it 'is false' do
        expect(address).not_to be_empty
      end
    end

    context 'when any of the main parts of the address are missing' do
      let(:address) do
        Pinpoint::Address.new(
          street:   nil,
          city:     'Nashville',
          state:    'Tennessee',
          zip_code: '37203',
          country:  'US',
        )
      end

      it 'is false' do
        expect(address).not_to be_empty
      end
    end

    context 'when all of the main parts of the address are present' do
      let(:address) do
        Pinpoint::Address.new(
          street:   '919 Broadway',
          city:     'Nashville',
          state:    'Tennessee',
          zip_code: '37203',
          country:  'US',
        )
      end

      it 'is false' do
        expect(address).not_to be_empty
      end
    end

    context 'when all of the main parts of the address are missing' do
      let(:address) do
        Pinpoint::Address.new(
          street:   nil,
          city:     nil,
          state:    nil,
          zip_code: nil,
          country:  nil,
        )
      end

      it 'is true' do
        expect(address).to be_empty
      end
    end
  end
end
