require 'spec_helper'
require 'pinpoint/format/style'

describe Pinpoint::Format::Style do
  let(:style_class) { Pinpoint::Format::Style }

  it 'can instantiate itself based on a style definition' do
    style_definition = '((%s, )(%l, ))(%p )%z(, %c)'
    style            = style_class.from_yaml style_definition

    expect(style.send(:structure).first.first).to eql [:street, ', ']
  end

  it 'can output an address based on a style' do
    style_definition = '((%s, )(%l, ))(%p )%z(, %c)'
    style            = style_class.from_yaml style_definition
    address          = double(street:      'a',
                              locality:    'b',
                              province:    'c',
                              postal_code: 'd',
                              country:     'e')

    expect(style.output(address)).to eql 'a, b, c d, e'
  end

  it 'can output an address based on a style with newlines' do
    style_definition = "((%s\n)((%l, ))(%p )%z\n)(%c\n)"
    style            = style_class.from_yaml style_definition
    address          = double(street:      'a',
                              locality:    'b',
                              province:    'c',
                              postal_code: 'd',
                              country:     'e')

    expect(style.output(address)).to eql <<-MULTILINE
a
b, c d
e
    MULTILINE
  end

  it 'can safely output unsafe HTML' do
    style_definition = '<span>%s</span>'
    style            = style_class.from_yaml style_definition
    address          = double(street: "<script>alert('Gotcha!');</script>")

    expect(style.output(address)).to eql '<span>&lt;script&gt;alert(&#39;Gotcha!&#39;);&lt;/script&gt;</span>'
  end

  it 'understands not to output a grouping without alphanumeric characters' do
    style_definition = '(%s, )'
    style            = style_class.from_yaml style_definition
    address          = double(street: '')

    expect(style.output(address)).to eql ''
  end
end
