require 'spec_helper'
require 'pinpoint/format/token_list'

describe Pinpoint::Format::TokenList do
  let(:token_list_class)  { Pinpoint::Format::TokenList }
  let(:token_list)        { token_list_class.new }
  let(:group_start_token) { double(type: :group_start) }
  let(:group_end_token)   { double(type: :group_end) }

  it 'is an Array' do; expect(token_list).to be_an Array; end

  it 'is valid if the number of group pair tokens is equal' do
    token_list = token_list_class.new [group_start_token, group_end_token]

    expect(token_list).to be_valid
  end

  it 'is not valid if the number of group pair tokens is unequal' do
    token_list = token_list_class.new [group_start_token]

    expect { token_list.valid? }.to raise_error Pinpoint::Format::UnevenNestingError
  end

  it 'yields each token to the block when processing' do
    result     = ''
    token_list = token_list_class.new [group_start_token, group_end_token]

    token_list.process_each! do |token|
      result << token.type.to_s
    end

    expect(result).to eql 'group_startgroup_end'
  end

  it 'deletes itself as it processes' do
    token_list = token_list_class.new [group_start_token]

    token_list.process_each! do |_token|
    end

    expect(token_list).to be_empty
  end
end
