require 'spec_helper'
require 'pinpoint/format'

describe Pinpoint::Format do
  let(:format_class) { Pinpoint::Format }
  let(:format)       { format_class.new }

  it 'can lookup a format by country' do
    allow(Pinpoint::Format::File).to receive(:styles_for).
                          and_return 'styles'

    format = format_class.lookup_by_country(:us)

    expect(format).to         be_a format_class
    expect(format.styles).to  eql 'styles'
  end

  context 'when it has one or more Styles' do
    let(:style) { double }

    before      { format.styles = { one_line: style } }

    it 'tells the relevant Style to output the address' do
      allow(style).to receive(:output).with('address').and_return 'formatted_address'

      expect(format.output('address', style: :one_line)).to eql 'formatted_address'
    end

    it 'defaults output style to one-line' do
      allow(style).to receive(:output).with('address').and_return 'formatted_address'

      expect(format.output('address')).to eql 'formatted_address'
    end
  end
end
